"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import urllib.parse
import html

class main:

		def __init__(self, q, limit=15):
			""" pubmed.ncbi.nlm.nih.gov search engine

					q         : query for search
					limit     : maximum result count
			"""
			self.framework = main.framework
			self.q = q
			self.max = limit
			self._rawhtml = ''
			self._articles = []
			self._links = []
			self._links_with_data = []
			self.pubmed = 'https://pubmed.ncbi.nlm.nih.gov'

		def run_crawl(self):
			self.q = urllib.parse.quote_plus(self.q)
			self.framework.verbose('Searching the pubmed domain...')

			url = f"https://pubmed.ncbi.nlm.nih.gov/?term={self.q}&size=200"
			try:
				req = self.framework.request(url=url)
			except Exception as e:
					self.framework.error(f"ConnectionError {e}.", 'util/pubmed', 'run_crawl')
					self.framework.error('Pubmed is missed!', 'util/pubmed', 'run_crawl')
					return
			self._rawhtml += req.text
			self._articles.extend(re.findall(r'<article .*?>(.*?)</article>', 
						req.text, 
						flags=re.DOTALL))

		@property
		def raw(self):
			return self._rawhtml

		@property
		def articles(self):
			return self._articles

		@property
		def links_with_data(self):
			findlink = lambda x: list(map(lambda x: self.pubmed+x, 
					re.findall(r'<a.*?class="docsum-title".*?href="(.*?)".*?>', x, flags=re.DOTALL)))
			findauthors = lambda x: re.findall(r'<span class="docsum-authors full-authors">(.*?)</span>', 
					x, flags=re.DOTALL)
			findtitle = lambda x: list(map(lambda x: html.unescape(re.sub('</?b>','',x)).strip(),
					re.findall(r'<a.*?class="docsum-title".*?>(.*?)</a>', x, flags=re.DOTALL)))

			for count,article in enumerate(self._articles):
					if count==self.max:
						break
					self._links_with_data.append({'authors':findauthors(article)[0],
							'title': findtitle(article)[0],
							'link' : findlink(article)[0]
							})

			return self._links_with_data



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import urllib.parse
import html

class main:

		def __init__(self, q, limit=15):
			""" arxiv.org search engine

					q         : query for search
					limit     : maximum result count
			"""
			self.framework = main.framework
			self.q = q
			self.max = limit
			self._rawxml = ''
			self._articles = []
			self._links = []
			self._links_with_data = []

		def run_crawl(self):
			self.q = urllib.parse.quote_plus(self.q)
			url = f'https://export.arxiv.org/api/query?search_query=all:'\
					   + f'{self.q}&start=0&max_results={self.max}'
			self.framework.verbose('[ARXIV] Searching the arxiv.org domain...')
			try:
					req = self.framework.request(url=url)
			except:
					self.framework.error('ConnectionError', 'arxiv', 'run_crawl')
					self.framework.error('ArXiv is missed!', 'arxiv', 'run_crawl')
					return
			self._rawxml = req.text
			self._articles = list(re.findall(r'<entry>(.*?)</entry>', 
					self._rawxml, 
					flags=re.DOTALL))

		@property
		def raw(self):
			return self._rawxml

		@property
		def articles(self):
			return self._articles

		@property
		def links(self):
			self._links = re.findall(r'(http://arxiv.org/abs/.*)<',
					self._rawxml)
			return self._links

		@property
		def links_with_data(self):
			findlink = lambda x: re.findall(r'(http://arxiv.org/abs/.*?)<', x)
			findauthors = lambda x: re.findall(r'<name>(.*?)</name>', x)
			findtitle = lambda x: re.findall(r'<title>(.*?)</title>', x, flags=re.DOTALL)

			for article in self._articles:
					self._links_with_data.append({'authors':findauthors(article),
							'title': list(map(html.unescape,findtitle(article))),
							'link' : findlink(article)
							})

			return self._links_with_data



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import html
from bs4 import BeautifulSoup as bs

class main:

	def __init__(self, q, limit=15):
		""" piratebay search engine
			q         : query for search
			limit     : maximum result count
		"""
		self.framework = main.framework
		self.q = self.framework.urlib(q).quote 
		self.max = limit
		self._rawhtml = ''
		self._torrents = []
		self._magnets = []
		self._rows = []
		self._links_with_data = []

	def run_crawl(self):
		url = f'https://tpb.party/search/{self.q}'
		self.framework.verbose('Searching piratebay...')
		try:
			req = self.framework.request(url=url)
		except:
			self.framework.error('ConnectionError.', 'util/piratebay', 'run_crawl')
			self.framework.error('Piratebay is missed!', 'util/piratebay', 'run_crawl')
			self.framework.error('Try again after a few seconds!', 'util/piratebay', 'run_crawl')
		else:
			soup = bs(req.text, 'html.parser')
			self._torrents = soup.find_all('tr')[1:-1]

	@property
	def raw(self):
		return self._rawhtml

	@property
	def links_with_data(self):
		findtitle = lambda x: x.find('a', {'class': 'detLink'}).text
		findmagnet =  lambda x: x.find('a', {'title': 'Download this torrent using magnet'})['href']
		finddateuploader = lambda x: x.find('font', {'class': 'detDesc'}).text
		seedandleech = lambda x: list(map(lambda x: x.text, 
		x.find_all('td', {'align': 'right'})))
		findlink = lambda x: x.find('a', {'class': 'detLink'})['href']

		for count,torrent in enumerate(self._torrents):
			if count == self.max:
				break

			title = findtitle(torrent)
			magnet = findmagnet(torrent)
			link = findlink(torrent)
			dateuploader = finddateuploader(torrent)
			seedleechcount = seedandleech(torrent)
		
			self._links_with_data.append({
				'title': html.unescape(title),
				'dateuploader': html.unescape(dateuploader),
				'magnet': magnet,
				'link': link,
				'seeders': seedleechcount[0],
				'leechers': seedleechcount[1]
				})

		return self._links_with_data
